@@ -1,61 +1,33 @@
 #' Navigation containers
 #'
-#' Render a collection of [nav()] items into a container.
+#' Render a collection of [nav_panel()] items into a container.
 #'
-#' @param ... a collection of [nav()] items.
-#' @param id a character string used for dynamically updating the container (see [nav_select()]).
-#' @param selected a character string matching the `value` of a particular [nav()] item to selected by default.
+#' @section Examples:
+#'
+#' ```{r child="man/fragments/ex-navset_tab.Rmd"}
+#' ```
+#'
+#' @param ... a collection of [nav_panel()] items.
+#' @param id a character string used for dynamically updating the container (see
+#'    [nav_select()]).
+#' @param selected a character string matching the `value` of a particular
+#'   [nav_panel()] item to selected by default.
 #' @param header UI element(s) ([tags]) to display _above_ the nav content.
 #' @param footer UI element(s) ([tags]) to display _below_ the nav content.
+#'
+#' @seealso [nav_panel()], [nav_panel_hidden()] create panels of content.
+#' @seealso [nav_menu()], [nav_item()], [nav_spacer()] create menus, items, or
+#'   space in the navset control area.
+#' @seealso [nav_insert()], [nav_remove()] programmatically add or remove nav
+#'   panels.
+#' @seealso [nav_select()], [nav_show()], [nav_hide()] change the state of a
+#'   [nav_panel()] in a navset.
+#'
+#' @family Panel container functions
+#' @rdname navset
+#' @name navset
 #' @export
-#' @seealso [nav()], [nav_select()].
-#' @rdname navs
-#' @examples
-#'
-#' library(shiny)
-#'
-#' nav_items <- function(prefix) {
-#'   list(
-#'     nav("a", paste(prefix, ": tab a content")),
-#'     nav("b", paste(prefix, ": tab b content")),
-#'     nav_item(
-#'       tags$a(icon("github"), "Shiny", href = "https://github.com/rstudio/shiny", target = "_blank")
-#'     ),
-#'     nav_spacer(),
-#'     nav_menu(
-#'       "Other links", align = "right",
-#'       nav("c", paste(prefix, ": tab c content")),
-#'       nav_item(
-#'         tags$a(icon("r-project"), "RStudio", href = "https://rstudio.com", target = "_blank")
-#'       )
-#'     )
-#'   )
-#' }
-#'
-#' if (interactive()) {
-#'   shinyApp(
-#'     page_navbar(
-#'       title = "page_navbar()",
-#'       bg = "#0062cc",
-#'       !!!nav_items("page_navbar()"),
-#'       footer = div(
-#'         style = "width:80%; margin: 0 auto",
-#'         h4("navs_tab()"),
-#'         navs_tab(!!!nav_items("navs_tab()")),
-#'         h4("navs_pill()"),
-#'         navs_pill(!!!nav_items("navs_pill()")),
-#'         h4("navs_tab_card()"),
-#'         navs_tab_card(!!!nav_items("navs_tab_card()")),
-#'         h4("navs_pill_card()"),
-#'         navs_pill_card(!!!nav_items("navs_pill_card()")),
-#'         h4("navs_pill_list()"),
-#'         navs_pill_list(!!!nav_items("navs_pill_list()"))
-#'       )
-#'     ),
-#'     function(...) { }
-#'   )
-#' }
-navs_tab <- function(..., id = NULL, selected = NULL,
+navset_tab <- function(..., id = NULL, selected = NULL,
                      header = NULL, footer = NULL) {
   tabs <- tabsetPanel_(
     ..., type = "tabs", id = id, selected = selected,
@@ -65,8 +37,8 @@ navs_tab <- function(..., id = NULL, selected = NULL,
 }
 
 #' @export
-#' @rdname navs
-navs_pill <- function(..., id = NULL, selected = NULL,
+#' @rdname navset
+navset_pill <- function(..., id = NULL, selected = NULL,
                       header = NULL, footer = NULL) {
   pills <- tabsetPanel_(
     ..., type = "pills", id = id, selected = selected,
@@ -75,10 +47,26 @@ navs_pill <- function(..., id = NULL, selected = NULL,
   as_fragment(pills)
 }
 
+#' @export
+#' @rdname navset
+navset_underline <- function(
+  ...,
+  id = NULL,
+  selected = NULL,
+  header = NULL,
+  footer = NULL
+) {
+  res <- tabsetPanel_(
+    ..., type = "underline", id = id, selected = selected,
+    header = header, footer = footer
+  )
+  as_fragment(res)
+}
+
 #' @export
 #' @inheritParams shiny::navlistPanel
-#' @rdname navs
-navs_pill_list <- function(..., id = NULL, selected = NULL,
+#' @rdname navset
+navset_pill_list <- function(..., id = NULL, selected = NULL,
                            header = NULL, footer = NULL,
                            well = TRUE, fluid = TRUE,
                            widths = c(4, 8)) {
@@ -92,8 +80,8 @@ navs_pill_list <- function(..., id = NULL, selected = NULL,
 }
 
 #' @export
-#' @rdname navs
-navs_hidden <- function(..., id = NULL, selected = NULL,
+#' @rdname navset
+navset_hidden <- function(..., id = NULL, selected = NULL,
                         header = NULL, footer = NULL) {
   hidden <- tabsetPanel_(
     ..., type = "hidden", id = id, selected = selected,
@@ -102,22 +90,64 @@ navs_hidden <- function(..., id = NULL, selected = NULL,
   as_fragment(hidden)
 }
 
-
 #' @inheritParams shiny::navbarPage
+#' @inheritParams page_fillable
+#' @param sidebar A [sidebar()] component to display on every [nav_panel()]
+#'   page.
+#' @param fillable Whether or not to allow `fill` items to grow/shrink to fit
+#'   the browser window. If `TRUE`, all [nav_panel()] pages are `fillable`. A
+#'   character vector, matching the `value` of [nav_panel()]s to be filled, may
+#'   also be provided. Note that, if a `sidebar` is provided, `fillable` makes
+#'   the main content portion fillable.
 #' @param bg a CSS color to use for the navbar's background color.
 #' @param inverse Either `TRUE` for a light text color or `FALSE` for a dark
 #'   text color. If `"auto"` (the default), the best contrast to `bg` is chosen.
 #' @export
-#' @rdname navs
-navs_bar <- function(..., title = NULL, id = NULL, selected = NULL,
+#' @rdname navset
+navset_bar <- function(..., title = NULL, id = NULL, selected = NULL,
+                     sidebar = NULL, fillable = TRUE,
+                     gap = NULL, padding = NULL,
                      # TODO: add sticky-top as well?
                      position = c("static-top", "fixed-top", "fixed-bottom"),
                      header = NULL, footer = NULL,
                      bg = NULL, inverse = "auto",
                      collapsible = TRUE, fluid = TRUE) {
+  padding <- validateCssPadding(padding)
+  gap <- validateCssUnit(gap)
+
+  navs_bar_(
+    ..., title = title, id = id, selected = selected,
+    sidebar = sidebar, fillable = fillable,
+    gap = gap, padding = padding,
+    position = position,
+    header = header, footer = footer,
+    bg = bg, inverse = inverse,
+    collapsible = collapsible, fluid = fluid,
+    # theme is only used to determine whether legacy style markup should be used
+    # (and, at least at the moment, we don't need legacy markup for this exported function)
+    theme = bs_theme()
+  )
+}
+
+# This internal version of navs_bar() exists so both it and page_navbar()
+# (and thus shiny::navbarPage()) can use it. And in the page_navbar() case,
+# we can use addition theme information as an indication of whether we need
+# to handle backwards compatibility
+navs_bar_ <- function(..., title = NULL, id = NULL, selected = NULL,
+                      sidebar = NULL, fillable = TRUE,
+                      gap = NULL, padding = NULL,
+                      position = c("static-top", "fixed-top", "fixed-bottom"),
+                      header = NULL, footer = NULL,
+                      bg = NULL, inverse = "auto",
+                      underline = TRUE,
+                      collapsible = TRUE, fluid = TRUE,
+                      theme = NULL) {
 
   if (identical(inverse, "auto")) {
     inverse <- TRUE
+    if (identical(theme_preset_info(theme)$name, "shiny")) {
+      inverse <- FALSE
+    }
     if (!is.null(bg)) {
       bg <- htmltools::parseCssColors(bg)
       bg_contrast <- bs_get_contrast(bs_theme("navbar-bg" = bg), "navbar-bg")
@@ -127,9 +157,12 @@ navs_bar <- function(..., title = NULL, id = NULL, selected = NULL,
 
   navbar <- navbarPage_(
     title = title, ..., id = id, selected = selected,
+    sidebar = sidebar, fillable = fillable,
+    gap = gap, padding = padding,
     position = match.arg(position),
     header = header, footer = footer, collapsible = collapsible,
-    inverse = inverse, fluid = fluid
+    inverse = inverse, underline = underline, fluid = fluid,
+    theme = theme
   )
 
   if (!is.null(bg)) {
@@ -142,8 +175,6 @@ navs_bar <- function(..., title = NULL, id = NULL, selected = NULL,
   as_fragment(navbar, page = page)
 }
 
-
-
 # -----------------------------------------------------------------------
 # 'Internal' tabset logic that was pulled directly from shiny/R/bootstrap.R
 #  (with minor modifications)
@@ -153,15 +184,18 @@ navbarPage_ <- function(title,
                        ...,
                        id = NULL,
                        selected = NULL,
+                       sidebar = NULL,
+                       fillable = TRUE,
+                       gap = NULL,
+                       padding = NULL,
                        position = c("static-top", "fixed-top", "fixed-bottom"),
                        header = NULL,
                        footer = NULL,
                        inverse = FALSE,
+                       underline = TRUE,
                        collapsible = FALSE,
                        fluid = TRUE,
-                       theme = NULL,
-                       windowTitle = title,
-                       lang = NULL) {
+                       theme = NULL) {
 
   # alias title so we can avoid conflicts w/ title in withTags
   pageTitle <- title
@@ -178,7 +212,10 @@ navbarPage_ <- function(title,
     selected <- shiny::restoreInput(id = id, default = selected)
 
   # build the tabset
-  tabset <- buildTabset(..., ulClass = "nav navbar-nav", id = id, selected = selected)
+  ulClass <- "nav navbar-nav"
+  if (underline)
+    ulClass <- paste(ulClass, "nav-underline")
+  tabset <- buildTabset(..., ulClass = ulClass, id = id, selected = selected)
 
   containerClass <- paste0("container", if (fluid) "-fluid")
 
@@ -230,13 +267,54 @@ navbarPage_ <- function(title,
       allTags()
   }
 
-  # build the main tab content div
-  contentDiv <- div(class = containerClass)
-  if (!is.null(header))
-    contentDiv <- tagAppendChild(contentDiv, div(class = "row", header))
-  contentDiv <- tagAppendChild(contentDiv, tabset$content)
-  if (!is.null(footer))
-    contentDiv <- tagAppendChild(contentDiv, div(class = "row", footer))
+  # If fillable is truthy, give the relevant .tab-content > .tab-pane containers
+  # the potential to fill
+  tabset$content <- makeTabsFillable(tabset$content, fillable, navbar = TRUE, gap = gap, padding = padding)
+
+  # For backwards compatibility reasons, wrap header & footer in a .row
+  # container if we're not using BS5+. I'm not entirely sure what the motivation
+  # was for it in the 1st place, but now that, with BS5, .row adds
+  # `display:flex` and makes children `width:100%`, which is surprising and
+  # confusing from a user perspective
+  isLegacy <- as.numeric(theme_version(theme) %||% 3) < 5
+  if (!is.null(header) && isLegacy) {
+    header <- div(class = "row", header)
+  }
+  if (!is.null(footer) && isLegacy) {
+    footer <- div(class = "row", footer)
+  }
+
+  contents <- dropNulls(list(header, tabset$content, footer))
+
+  if (is.null(sidebar)) {
+
+    contentDiv <- div(class = containerClass, !!!contents)
+
+    # If fillable is truthy, the .container also needs to be fillable
+    if (!isFALSE(fillable)) {
+      contentDiv <- bindFillRole(contentDiv, container = TRUE, item = TRUE)
+    }
+
+  } else {
+
+    contentDiv <- div(
+      # In the fluid case, the sidebar layout should be flush (i.e.,
+      # the .container-fluid class adds padding that we don't want)
+      class = if (!fluid) "container",
+      layout_sidebar(
+        sidebar = sidebar,
+        fillable = !isFALSE(fillable),
+        border_radius = FALSE,
+        border = !fluid,
+        contents
+      )
+    )
+
+    # Always have the sidebar layout fill its parent (in this case
+    # fillable controls whether the _main_ content portion is fillable)
+    contentDiv <- bindFillRole(contentDiv, container = TRUE, item = TRUE)
+
+  }
 
   # *Don't* wrap in bootstrapPage() (shiny::navbarPage()) does that part
   tagList(
@@ -254,7 +332,7 @@ navbarMenu_ <- function(title, ..., menuName = title, icon = NULL, align) {
       tabs = list2(...),
       # Here for legacy reasons
       # https://github.com/cran/miniUI/blob/74c87d3/R/layout.R#L369
-      iconClass = tagGetAttribute(icon, "class"),
+      iconClass = if (inherits(icon, "shiny.tag")) tagGetAttribute(icon, "class"),
       icon = icon,
       align = align
     ),
@@ -274,7 +352,7 @@ tabPanel_ <- function(title, ..., value = title, icon = NULL) {
     `data-value` = value,
     # Here for legacy reasons
     # https://github.com/cran/miniUI/blob/74c87d/R/layout.R#L395
-    `data-icon-class` = tagGetAttribute(icon, "class"),
+    `data-icon-class` = if (inherits(icon, "shiny.tag")) tagGetAttribute(icon, "class"),
     ...
   )
   attr(pane, "_shiny_icon") <- icon
@@ -302,7 +380,7 @@ tabPanelBody_ <- function(value, ..., icon = NULL) {
 tabsetPanel_ <- function(...,
                         id = NULL,
                         selected = NULL,
-                        type = c("tabs", "pills", "hidden"),
+                        type = c("tabs", "pills", "hidden", "underline"),
                         header = NULL,
                         footer = NULL) {
 
@@ -399,21 +477,12 @@ findAndMarkSelectedTab <- function(tabs, selected, foundSelected) {
 }
 
 prepTabIcon <- function(x = NULL) {
-  if (is.null(x)) return(NULL)
-  if (!inherits(x, "shiny.tag")) {
-    stop(
-      "`icon` must be a `shiny.tag` object. ",
-      "Try passing `icon()` (or `tags$i()`) to the `icon` parameter.",
-      call. = FALSE
-    )
-  }
+  if (!inherits(x, "shiny.tag")) return(x)
 
   is_fa <- grepl("fa-", tagGetAttribute(x, "class") %||% "", fixed = TRUE)
-  if (!is_fa) {
-    return(x)
-  }
+  if (!is_fa) return(x)
 
-  # for font-awesome we specify fixed-width
+  # specify fixed-width for font-awesome
   tagAppendAttributes(x, class = "fa-fw")
 }
 
@@ -497,7 +566,7 @@ buildTabItem <- function(index, tabsetId, foundSelected, tabs = NULL,
 
   # The behavior is undefined at this point, so construct a condition message
   msg <- paste0(
-    "Navigation containers expect a collection of `bslib::nav()`/`shiny::tabPanel()`s ",
+    "Navigation containers expect a collection of `bslib::nav_panel()`/`shiny::tabPanel()`s ",
     "and/or `bslib::nav_menu()`/`shiny::navbarMenu()`s. ",
     "Consider using `header` or `footer` if you wish to place content above ",
     "(or below) every panel's contents."
